import { select, json, xml, tree, event, hierarchy, zoom, linkHorizontal } from 'd3';

xml('data.xml')
  .then(data  => {

  var svg = d3.select("body").append("svg")
      .attr("width", document.body.clientWidth)
      .attr("height", document.body.clientHeight);

  var nodes = [];
  var links = [];
  var contents = {};
  var root;
  var width = 960, height = 500;
  
  function Traverse(d, parent) {
      if (d === null) return;
      if (d.nodeName === 'richcontent') return;
      if (d.nodeName === 'node') {
          d.id = d.getAttribute('ID');
          d.lotid = d.getAttribute('CONTENT_ID');
          d.created = d.getAttribute('CREATED');
          d.tree_id = d.getAttribute('TREE_ID');
          contents[d.id] = d;
          if (d.id && !parent) {
            root = d;
//          root.fx = 30;
//          root.fy = height / 2;
            d.isroot = true;
          }
          if (d.id && !d.lotid)
              nodes.push(d);
          if (parent && parent.id && !parent.lotid) {
              if (contents[d.lotid])
                contents[d.lotid].iscloned = true;
              links.unshift({
                  "source": parent.id,
                  "target": d.lotid ? d.lotid : d.id,
                  "isclone": d.lotid ? true : false
              });
          }
      }
      for (let c = d.firstElementChild; c != null; c = c.nextElementSibling)
          Traverse(c, d.id ? d : parent);
      return;
  }

    Traverse(data, null);

    function force(alpha) {
      for (var i = 0, n = nodes.length, node, k = alpha * 0.15; i < n; ++i) {
        node = nodes[i];
        node.vx -= node.x * k;
        node.vy -= node.y * k;
      }
    }

    var graph = { "nodes": nodes, "links" : links };  
    var simulation = d3.forceSimulation()
            .nodes(graph.nodes);                    

        simulation
            .force("center_force", d3.forceCenter(width / 2, height / 2))
            .force("charge_force",
                d3.forceManyBody()
                   .theta(0.8)
                   .strength(-100)
             )
//          .force("root_force", d3.forceManyBody([root]).strength(10))
            .force("links",
                d3.forceLink(graph.links)
                  .id(d => d.id)
                  .distance(link => link.isroot ? 100 : link.isclone ? 50 : 30)
                  .strength(link => link.isroot ? 1.0 : link.isclone ? 0.8 : 0.2)
             )
//          .force("y_force",
//              d3.forceY([root])
//                 .y(50)
//                 .strength(0.15)
//           )
//          .force("y_force",
//              d3.forceY(graph.links)
//                 .y(d => d.iscloned ? height / 2 : height / 2)
//                 .strength(d => d.iscloned ? 0.70 : 0)
//           )
            .force("collide", d3.forceCollide().radius(18))
//          .force("custom", force)
            ;         

        simulation
            .on("tick", ticked);

        //add encompassing group for the zoom 
        var g = svg.append("g")
            .attr("class", "everything");

        //Create deffinition for the arrow markers showing relationship directions
        g.append("defs").append("marker")
            .attr("id", "arrow")
            .attr("viewBox", "0 -3 10 10")
            .attr("refX", 20)
            .attr("refY", 0)
            .attr("markerWidth", 8)
            .attr("markerHeight", 8)
            .attr("orient", "auto")
            .append("svg:path")
            .attr("d", "M0,-5L10,0L0,5");        

        var link = g.append("g")
            .attr("class", "links")
            .selectAll("line")
            .data(graph.links)
            .enter().append("line")
            .attr("stroke",       d => d.isclone ? 'red' : 'black')
            .attr('stroke-width', d => d.isclone ? 2 : 1)
            .attr('stroke-opacity', 0.5)
            .attr("marker-end", "url(#arrow)");

        var node = g.append("g")
            .attr("class", "nodes")
            .selectAll("circle")
            .data(graph.nodes)
            .enter()
            .append("circle")
            .attr("r",       d => d.isroot ? 14 : 7)
            .attr("fill",    d => d.isroot ? 'darkblue' : d.iscloned ? 'orangered' : 'purple')
            .attr("stroke",  d => d.isroot ? 'darkblue' : d.iscloned ? 'black' : 'purple')
            .attr('stroke-width', d => d.iscloned ? 3 : 1);

        var drag_handler = d3.drag()
            .on("start", drag_start)
            .on("drag", drag_drag)
            .on("end", drag_end); 

        drag_handler(node);

        var text = g.append("g").attr("class", "labels").selectAll("g")
            .data(graph.nodes)
            .enter().append("g")
            .append("text")
            .attr("x", 12)
            .attr("y", ".31em")
            .style("font-family", "sans-serif")
            .style("font-size", 12 /*"0.9em"*/)
            .text(function (d) { return d.id; });

        node.on("click", function (d) {
            d3.event.stopImmediatePropagation();
            self.onNodeClicked.emit(d.id);
        });

        node.append("title")
            .text(function (d) { return d.lotid; });
        
        //add zoom capabilities 
        var zoom_handler = d3.zoom()
            .on("zoom", zoom_actions);

        zoom_handler(svg); 

        //Drag functions 
        //d is the node 
        function drag_start(d) {
            if (!d3.event.active) simulation.alphaTarget(0.3).restart();
            d.fx = d.x;
            d.fy = d.y;
        }
        
        //make sure you can't drag the circle outside the box
        function drag_drag(d) {
            d.fx = d3.event.x;
            d.fy = d3.event.y;
        }
        
        function drag_end(d) {
            if (!d3.event.active) simulation.alphaTarget(0);
            d.fx = null;
            d.fy = null;
        }
        
        function zoom_actions(){
            g.attr("transform", d3.event.transform)
        }
        
        function ticked() {
            //update circle positions each tick of the simulation 
            node
                .attr("cx", function(d) { return d.x; })
                .attr("cy", function(d) { return d.y; });
                
            //update link positions 
            link
                .attr("x1", function(d) { return d.source.x; })
                .attr("y1", function(d) { return d.source.y; })
                .attr("x2", function(d) { return d.target.x; })
                .attr("y2", function(d) { return d.target.y; });

            text
                .attr("transform", function (d) { return "translate(" + d.x + "," + d.y + ")"; });
        }
});
